
#include "sparseData.h"
#include "solver.h"
#include "kernel.h"
#include "svm.h"
#include "budgetMaintenance.h"

#include <iostream>
#include <string>
#include <tuple>
#include <cmath>

using namespace std;


void help(string const& programname)
{
	cout << endl
		<< "Usage:" << endl
		<< programname << " [parameters]" << endl
		<< "The following parameters are recognized:" << endl
		<< "  --train <trainingdata>       training data in sparse format" << endl
		<< "  --test <testdata>            test data in sparse format" << endl
		<< "  --method <method>            GSS, GSSprecise, LookupH, or LookupWD" << endl
		<< "  --epochs <number>            number of passes through the training data" << endl
		<< "  --C <number>                 C parameter of the SVM" << endl
		<< "  --gamma <number>             gamma parameter of the Gaussian kernel" << endl
		<< "  --B <number>                 budget size, default=1" << endl
		<< "  --seed <number>              random number generator seed for SGD, default=1" << endl
		<< endl
		<< "All parameters except for --test, --epochs, and --seed are required." << endl
		<< endl;
	exit(EXIT_FAILURE);
}

int main(int argc, char** argv)
{
	string train_path;
	string test_path;
	double C = -1;
	double gamma = -1;
	Method method;
	bool method_set = false;
	size_t epochs = 1;
	size_t budget = 0;
	size_t seed = 1;

	// parse command line parameters
	vector<string> arguments(argv + 1, argv + argc);
	for (size_t i=0; i<arguments.size(); i++) {
		if (arguments[i] == "--train") {
			i++;
			if (i >= arguments.size()) help(argv[0]);
			train_path = arguments[i];
		} else if (arguments[i] == "--test") {
			i++;
			if (i >= arguments.size()) help(argv[0]);
			test_path = arguments[i];
        } else if (arguments[i] == "--method") {
			i++;
			if (i >= arguments.size()) help(argv[0]);
			if (arguments[i] == "GSS") method = GSSstandard;
			else if (arguments[i] == "GSSprecise") method = GSSprecise;
			else if (arguments[i] == "LookupH") method = LookupH;
			else if (arguments[i] == "LookupWD") method = LookupWD;
			else help(argv[0]);
			method_set = true;
		} else if(arguments[i] == "--epochs") {
			i++;
			if (i >= arguments.size()) help(argv[0]);
			epochs = stoi(arguments[i]);
		} else if (arguments[i] == "--C") {
			i++;
			if (i >= arguments.size()) help(argv[0]);
            C = stod(arguments[i]);
		} else if (arguments[i] == "--gamma") {
			i++;
			if (i >= arguments.size()) help(argv[0]);
            gamma = stod(arguments[i]);
		} else if (arguments[i] == "--B") {
			i++;
			if (i >= arguments.size()) help(argv[0]);
			budget = stoi(arguments[i]);
		} else if (arguments[i] == "--seed") {
			i++;
			if (i >= arguments.size()) help(argv[0]);
			seed = stoi(arguments[i]);
		} else {
			cout << "WARNING: unrecognized parameter " << arguments[i] << endl;
		}
	}

	// check for validity
	if (! method_set || C <= 0.0 || gamma <= 0.0 || budget == 0 || train_path.empty()) help(argv[0]);

	// load training data
	sparseData train_data;
    cout << "Loading training dataset ... " << flush;
	loaddata(train_path, train_data);
	cout << "complete." << endl;

	// load test data if provided
	sparseData test_data;
	if (! test_path.empty())
	{
	    cout << "Loading test dataset ... " << flush;
		loaddata(test_path, test_data);
		cout << "complete." << endl;
	}

	// dump the configuration
	vector<string> methodname { "GSS", "GSSprecise", "LookupH", "LookupWD" };
    cout << endl;
	cout << "training data: " << train_path << "   " << train_data.data.size() << " points" << endl;
	if (! test_path.empty()) cout << "test data: " << test_path << "   " << test_data.data.size() << " points" << endl;
	cout << "budget=" << budget << endl;
	cout << "epochs=" << epochs << endl;
	cout << "method=" << methodname[(int)method] << endl;
	cout << "seed=" << seed << endl;
    cout << "C=" << C << endl;
    cout << "gamma=" << gamma << endl;
    cout << endl;

	// train the model
	RbfKernel kernel(gamma);
	double train_start_time = (double)clock() / CLOCKS_PER_SEC;
	SVM svm = BSGD(train_data, C, kernel, budget, epochs, seed, method);
	double train_end_time = (double)clock() / CLOCKS_PER_SEC;
	cout << "training time: " << (train_end_time - train_start_time) << " seconds" << endl;

	// test the model
	if (! test_path.empty())
	{
		double test_start_time = (double)clock() / CLOCKS_PER_SEC;
		double performance = svm.evaluateTestset(test_data);
		double test_end_time = (double)clock() / CLOCKS_PER_SEC;
		cout << "test time: " << (test_end_time - test_start_time) << " seconds" << endl;
		cout << "test accuracy: " << performance << endl;
	}

	return EXIT_SUCCESS;
}
