#include "sparseData.h"
#include "kernel.h"
#include "svm.h"
#include "budgetMaintenance.h"

#include <vector>
#include <algorithm>
#include <cmath>
#include <random>

using namespace std;


void downgrade(unsigned long numIter, vector<double>& pseudo_variables)
{
	double factor = 1.0 - 1.0 / (double)numIter;
	for (double& a : pseudo_variables)
		a *= factor;
};

double computeMargin(vector<SE>& point, char label, vector<double>& pseudo_variables, sparseData const& pseudo_data, Kernel const& kernel) {
	double fx = 0;
	size_t number_of_variables = pseudo_variables.size();
	for (INDEX j = 0; j < number_of_variables; j++) {
		fx += pseudo_variables[j] * pseudo_data.labels[j] * kernel.evaluate(point, pseudo_data.data[j]);
	}
	return 1 - 2 * label * fx;
}

//Budgeted primal solver
SVM BSGD(sparseData& dataset, double C, Kernel const& kernel, size_t B, size_t epochs, size_t seed, Method method)
{
	default_random_engine rng(seed);

	size_t number_of_training_points = dataset.data.size();
	double lambda = 1.0 / ( (double)number_of_training_points * C);

	sparseData pseudo;
	vector<double> pseudo_variables;

	vector<INDEX> sequence(number_of_training_points);

	unsigned int numIter = 0;
	for (size_t epoch = 0; epoch < epochs; epoch++)
	{
		// loop through the training set in random order
		iota(sequence.begin(), sequence.end(), 0);
		shuffle(sequence.begin(), sequence.end(), rng);
		for (INDEX i = 0; i < number_of_training_points; i++)
		{
			INDEX ws = sequence[i];
			numIter++;

			/* calculation the margin (line 4 in algorithm 1)*/
			double marginViolation = computeMargin(dataset.data[ws], dataset.labels[ws], pseudo_variables, pseudo, kernel);

			/* calculation the downgrade step (line 5 in algorithm 1)*/
			downgrade(numIter, pseudo_variables);

			/* line 6 in algorithm 1*/
			if ( (marginViolation) > 0.0)  // check margin violation
			{
				//add a new SV to the pseudo model (line 7 in algorithm 1)
				pseudo_variables.push_back( (1.0 / ((double)numIter * lambda)));
				pseudo.data.push_back(dataset.data[ws]);
				pseudo.labels.push_back(dataset.labels[ws]);
			}
			/* Check the model size compared to the budget (line 8 in algorithm 1)*/
			while (pseudo.data.size() > B)
			{
				/* Check the model size compared to the budget (line 9 in algorithm 1)*/
				mergeAndDeleteSV(pseudo_variables, pseudo, kernel, C, method);
			}
		}
	}
	return SVM(pseudo_variables, pseudo, kernel);
}
